#! /usr/bin/perl

foreach $_ (split(/:/,$ENV{'PATH'})) {
   push(@INC, $_) if -f "$_/tfm.pl";
}
require 'tfm.pl';


$tnow = 0;
$fnow = 0;
$fps = 5;
@bgcolor = (0.3, 0.3, 0.3);
@upvector = (0,0,1);
@center = (0,0,0);
@lc2w = &tfm(1);
@lvd = (0)x6;

sub Usage {
    print STDERR <<EOF;
Usage: $0 [-t] [-f fps] [-o outfmt | -play] [starttime [endtime]]
Emit partiview commands to drive (and with -o, record images from)
an animation script.  Use as in
    $0 -o /some/where/snap.%04d.ppm  > driver.cmds
    partiview -C driver.cmds  some-scene.cf
EOF
    exit(1);
}

while($ARGV[0] =~ /^-+\w/) {
    $_ = shift(@ARGV);
    $fps = shift, next if /^-f/;
    $recordfmt = shift, next if /^-o/;
    $play = 1, next if /^-+p/;
    print STDERR "Unknown option: $_\n" unless /^-+[h?]/;
    &Usage;
}

if(!defined($recordfmt) && !defined($play)) {
    print STDERR "Must specify either -o <output-image-template>   or  -p (just-play-without-recording)\n";
    print STDERR "\n";
    &Usage;
}

if($recordfmt) {
    $recordfmt .= ".%04d.png" if $recordfmt !~ /\%/;
}

$starttime = $ARGV[0];
$endtime = $ARGV[1];

$fprev = "";

&say("snapset $recordfmt") if $recordfmt;

sub say {
  my($fnow) = $fnow;

  return if(defined($endtime) && $fnow > $endtime);
  if(defined($starttime) && $fnow < $starttime) {
    return if $_[0] =~ /^jump/;
    $fnow = $starttime;
  }

  if($recordfmt) {
    # write recording commands
    if($_[0] =~ /^(#|\s*)$/) {
	print join(" ", @_), "\n";
	return;
    }
    if($fnow ne $fprev) {
	if($fprev ne "") {
	    printf "eval snapshot -n %d\n", $fprev;
	    my($fref) = $fprev;
	    while(++$fprev < $fnow) {
		printf "eval snapshot -n %d ## duplicate of %d\n", $fprev, $fref;
	    }
	}
    }
    $fprev = $fnow;
  } else {
    print "eval update\n";
  }
  print "eval ", join(" ", @_), "\n";
}

sub turnon {
  local(@gs) = @_;
  local($g);
  local($set) = "on";
  foreach $g (@gs) {
    if($g =~ /^g?(-?)g?(\d+)/) {
	&say( $1 eq "-" ? "g$2 off" : "g$2 $set" );
    } elsif($g =~ /on|off/) {
	$set = $g;
    }
  }
}

sub hold {
  local(@nowthen) = &frames($_[0]);
  $tnow += $_[0];
  ($fnow) = &frames(0);
  printf "# %d..%d hold %d\n", @nowthen, $_[0];
  &say("update");
}

sub jump {
  @_ = split(' ', $_[0]) if @_ == 1;
  if(@_ == 16) {
    @lc2w = @_;
    @lvd = &tfm2vd( @_ );
  } elsif(@_ == 6) {
    @lvd = @_;
    @lc2w = &vd2tfm( @_ );
  } else {
    print STDERR "jump: got ", (0+@_), " args not 6 nor 16!\n";
  }
  &say( sprintf("jump %.7g %.7g %.7g  %.6f %.6f %.6f", @lvd ) );
}

sub center {
  @center = @_;
  @center = (split(' ', $_[0]))[0..2] if(@center == 1);
  &say("center", @center);
  &say("update");
}

sub interest { &center; }

# &bgcolor(newlevel [, nseconds])
sub bgcolor {
  local($newbg, $dt) = @_;
  local(@bgc) = split(' ', $newbg);
  local($smooth) = ($dt =~ /s/);

  push(@bgc, @bgc, @bgc) if @bgc == 1;
  local($ffrom, $fto) = &frames($dt);
  local($f);

  printf "# %d..%d bgcolor %.3f %.3f %.3f  (was %.3f %.3f %.3f)\n",
	$ffrom, $fto, @bgc, @bgcolor;
  for($f = $ffrom; $f <= $fto; $f++) {
    $frac = $fto>$ffrom ? ($f - $ffrom) / ($fto - $ffrom) : 1.0;
    $frac *= $frac*(3-2*$frac) if $smooth;
    $fnow = $f;
    &say( sprintf( "bgcolor %.3f %.3f %.3f",
		&lerp( $frac, @bgcolor, @bgc ) ) );
  }
  @bgcolor = @bgc;
  $tnow += $dt;
}
  

sub frames {
  ( int($tnow*$fps + 0.5), int(($tnow + $_[0]) * $fps + 0.5) );
}

  
# &pqlerp(frac, place1[3], quat1[4], place2[3], quat2[4])
sub pqlerp {
  local($frac) = shift(@_);
  # p1 is 0..2   p2 is 7..9
  # q1 is 3..6   q2 is 10..13
  warn "&pqlerp expects frac, p1[3],q1[4], p2[3],q2[4]; got " . (0+@_) . " parameters instead."
	if @_ != 14;

  ( &lerp( $frac, @_[0..2], @_[7..9] ),
    &normalize( &lerp( $frac, @_[3..6], @_[10..13] ) )
  );
}

sub vd2pq {
  local(@tfm) = (@_ == 16) ? @_
	: &vd2tfm( split(' ', join(" ", @_)) );

  ( @tfm[12..14], t2quat(@tfm) );
}

sub pq2tfm {
  local(@tfm) = &quat2t( @_[3..6] );
  @tfm[12..14] = @_[0..2];
  @tfm;
}


# &leapto(nseconds, new-jump-position)
sub leapto {
  local($t0) = $tnow;
  local($dt, @pos2) = @_;
  local($smooth) = ($dt =~ /s/);
  local($ffrom, $fto) = &frames($dt);

printf "\n# %d..%d leapto %s\n", $ffrom, $fto, join(" ", @_);

  local(@pqpq) = ( &vd2pq(@lc2w), &vd2pq(@pos2) );
  if(&dot(@pqpq[3..6], @pqpq[10..13]) < 0) {
    # If quaternions are in opposite hemispheres, flip one of them.
    @pqpq[10..13] = &svmul( -1, @pqpq[10..13] );
  }

  local(@leapc2w);
  local($f, $frac);
  for($f = $ffrom; $f <= $fto; $f++) {
    $frac = $fto>$ffrom ? ($f - $ffrom) / ($fto - $ffrom) : 1.0;
    $frac *= $frac*(3-2*$frac) if $smooth;
    @leapc2w = &pq2tfm( &pqlerp( $frac, @pqpq ) );
    $fnow = $f;
    &jump( @leapc2w );
  }
  $tnow += $dt;
}


# &cenrot( c2w0, ndegrees, axis)
sub cenrot {
  local(@c2w0) = splice(@_, 0, 16);
  local($angle, @axis) = @_;

  local(@w2c0) = &eucinv( @c2w0 );

  local($camaxis) = ($axis[0] =~ /^c/i), shift(@axis)	# "camera" or "world"
	if $axis[0] =~ /^[cw]/i;
  local($isquat) = ($angle =~ /^q/i);			# "quat" or angle
  local(@rot) = $isquat ? &quat2t(@axis) : &tfm(@axis, $angle);
  local(@c2w1);
  if($camaxis) {
    @c2w1 = &tmul( @rot, @c2w0 );	# Rotation in camera coordinates
  } else {
    @c2w1 = &tmul( @c2w0, @rot );	# Rotation in world coordinates
  }
  local(@vx) = @c2w1[0..2];
  local(@vy) = @c2w1[4..6];
  local($rx) = &dot( @upvector, @vx );
  local($ry) = &dot( @upvector, @vy );
  if($rx*$rx > 1e-7) {
    # Maintain up-vector
    ($rx, $ry) = &normalize($rx, $ry);
    @c2w1[0..2] = &vcomb( $ry, @vx, -$rx, @vy );
    @c2w1[4..6] = &vcomb( $rx, @vx,  $ry, @vy );
# Check: we should now have that &dot( @upvector, @c2w1[0..2] ) is nearly zero.
    $zerocheck = &dot( @upvector, @c2w1[0..2] );
  }

  # Get world coordinates of where this rotation put the "center"
  local(@wantcen) = &v3mmul( &v3mmul( @center, @w2c0 ), @c2w1 );

  # Translate @c2w1 to put @wantcen back where @center says it should stay
  local(@wdcen) = &vsub( @center, @wantcen );
  @c2w1[12..14] = &vsadd( 1,@wdcen, @c2w1[12..14] );

  @c2w1;
}

# &spin(nseconds, ndegrees, ["camera"], axisname-or-3vector)
sub spin {
  local($dt, $angle, @axis) = @_;
  local($smooth) = ($dt =~ /s/);
  local($ffrom, $fto) = &frames( $dt );
  local($f, $frac, @rc2w);
  local(@c2w0) = @lc2w;


  printf "# %d..%d spin %s %s %s\n",
	$ffrom,$fto, $dt, $angle, join(" ", @axis);
  for($f = $ffrom+1; $f <= $fto; $f++) {
    $frac = $fto>$ffrom ? ($f - $ffrom) / ($fto - $ffrom) : 1.0;
    $frac *= $frac*(3-2*$frac) if $smooth;
    @rc2w = &cenrot( @c2w0, $angle*$frac, @axis );
    $fnow = $f;
    &jump( @rc2w );
  }
  $tnow += $dt;
}

# &hulaloop(nseconds, coneangle)
sub hulaloop {
  # Perform a cycle of rotations -- starting from zero and returning to zero --
  # which form a loop (cardioid, circle, ...) in quaternion space.
  # Easiest to construct these by just writing quaternion components
  # for a camera-coordinate rotation.
  local($dt, $coneangle) = @_;
  local($ffrom, $fto) = &frames($dt);
  local($smooth) = ($dt =~ /s/);
  local($rloop) = sin(rad($coneangle)/2);

  printf "\n# %d..%d hulaloop %s %s\n", $ffrom,$fto, $dt, $coneangle;

  local(@c2w0) = @lc2w;
  local(@rc2w);
  local(@q) = (0,0,0);
  local($f, $frac, $theta);
  for($f = $ffrom+1; $f <= $fto; $f++) {
    $frac = $fto>$ffrom ? ($f - $ffrom) / ($fto - $ffrom) : 1.0;
    $frac *= $frac*(3-2*$frac) if $smooth;
    $theta = $frac * 2*$pi;
    @q = ($rloop*(sin($theta)), $rloop*.6*sin(2*$theta), 0);
    @rc2w = &cenrot( @c2w0, "quat", "camera", @q );
    $fnow = $f;
    &jump( @rc2w );
  }
  $tnow += $dt;
}

1;
