#! /usr/bin/env python

import numpy

# Make some data - four variables defined on a sphere

attrnames = ['lon', 'lat', 'simplefunc', 'wildfunc']

# longitudes, from 0 to 2pi
lons = numpy.linspace( 0, 2*numpy.pi, num=120, endpoint=False )

# latitudes, from -pi/2 to pi/2
lats = numpy.linspace( -0.5*numpy.pi, 0.5*numpy.pi, num=60, endpoint=True )

# make space for grid of points and attributes
npts = len(lons) * len(lats)
nattrs = len(attrnames)

xyz = numpy.zeros( shape=(npts, 3) )		      # particle positions
ids = numpy.zeros( shape=(npts,), dtype=numpy.int )   # integer particle ids, even though we don't need them here
attrs = numpy.zeros( shape=(npts, nattrs) )	      # particle attributes


outf = open("sphere-example.speck", "w")

# Write a typical .speck header which gives names to the attributes below
print >>outf, "#! /usr/bin/env partiview"
print >>outf, "datavar 0 lon"
print >>outf, "datavar 1 lat"
print >>outf, "datavar 2 simplefunc"
print >>outf, "datavar 3 wildfunc"

for lat in lats:
    for lon in lons:
	x = numpy.cos(lon) * numpy.cos(lat)
	y = numpy.sin(lon) * numpy.cos(lat)
	z = numpy.sin(lat)
	simple = numpy.cos( 3 * lon )	# simple function
	wild = numpy.cos( 3 * lon + 3 * numpy.tan( 0.9*lat ) )  # wilder function
	print >>outf, "%g %g %g %g %g %g %g" % (x,y,z, lon,lat,simple,wild)

outf.close()

## Helpfully write out a wrapper script which reads the speck file
print "#! /usr/bin/env partiview"
print "# Paste this into a script, e.g. sphere-example.cf, and run: partiview sphere-example.cf"
print "read sphere-example.speck"
print "eval lum const 5 # set a default brightness"
print "eval color wildfunc -1.01 1.01 # slightly wider data-range than wildfunc's actual values"
print "# Select various views by typing"
print "#   \"color wildfunc\" or \"color simplefunc\" or \"color lon\" or \"color lat\""
print "eval jump -2.5 -5.3 3.5  51 -36 -38 # Nice initial viewpoint.   Determined by hand, then typed \"jump\""

